import org.apache.commons.httpclient.*
import org.apache.commons.httpclient.methods.*;
import org.apache.commons.httpclient.protocol.*;
import org.apache.commons.httpclient.auth.AuthScope;
import com.urbancode.commons.util.https.OpenSSLProtocolSocketFactory;


final def isWindows = (System.getProperty('os.name') =~ /(?i)windows/).find()
final def ahptool = isWindows ? 'ahptool.cmd' : 'ahptool'
final def out = System.out

final def getStepProperties = {
    def props = [:]

    def ahptoolProc = [ahptool, "getStepProperties"].execute()
    ahptoolProc.consumeProcessErrorStream(System.out);
    def propertiesXml = ahptoolProc.text
    ahptoolProc.waitFor()
    if (ahptoolProc.exitValue()) {
        throw new Exception("Failed to get step properties from ahptool: " + propertiesXml)
    }
    new XmlSlurper().parseText(propertiesXml).property.each{ propElem ->
        props[propElem.@name.text()] = propElem.text()
    }

    return props;
}

//-------------------------------------------------------------------------
// Scripts variables
//=========================================================================

final def props = getStepProperties()


System.out.println "Server:\t"+props['integration/rallyUrl']
System.out.println "State:\t"+ props['newState']
System.out.println "Defect:\t"+ props['defectKey']
System.out.println "Type:\t" + props['type'];

if (props['defectKey'] && props['defectKey'].trim().length() > 0) {
    // Generate XML doc
    def writer = new StringWriter()
    def builder = new groovy.xml.MarkupBuilder(writer)
    if("Defect".equals(props['type'])) {
    	builder.Defect(){
    		State(props['newState'])
    	}
    }
    else if("Task".equals(props['type'])) {
    	builder.Task(){
    		State(props['newState'])
    	}
    }


    //get rally props for GETS and POSTS to rally
    String base_url = new URL(props['integration/rallyUrl']).getHost()
    String url = props['integration/rallyUrl']+"/slm/webservice/"+props['integration/rallyVer']+"/"
    String user = props['integration/rallyUser']
    String password = props['integration/rallyPass']?props['integration/rallyPass']:props['integration/passScript']
	String proxyHost = props['integration/proxyHost']
	String proxyPort = props['integration/proxyPort']
	String proxyPass = props['integration/proxyPass']
	String proxyUser = props['integration/proxyUser']

    // create and setup httpClient

    HttpClient client = new HttpClient()
    client.getParams().setAuthenticationPreemptive(true)
    Credentials defaultcreds = new UsernamePasswordCredentials(user, password)
    client.getState().setCredentials(new AuthScope(base_url, 443), defaultcreds)
    
    ProtocolSocketFactory socketFactory = new OpenSSLProtocolSocketFactory()
    Protocol https = new Protocol("https", socketFactory, 443)
    Protocol.registerProtocol("https", https)
	
	// setup proxy if neccessary
	if (proxyHost) {
		out.println("Setting up proxy")
		client.getHostConfiguration().setProxy(proxyHost, Integer.valueOf(proxyPort));
		if (proxyUser && proxyPass) {
			client.getState().setProxyCredentials(AuthScope.ANY,
					new UsernamePasswordCredentials(proxyUser, proxyPass));
		}
	}



    def final getWorkspace = {
        HttpMethod workspaceMethod = null
        try {
            workspaceMethod= new GetMethod(url + "subscription")
            workResult = client.executeMethod(workspaceMethod)
            workResponse = workspaceMethod.getResponseBodyAsString()
            workXml = new XmlParser().parseText(workResponse)
            return workXml.Workspaces.Workspace.findAll{it.'@refObjectName'.equals(props['integration/workspace']) }[0].'@ref'
        }
        finally {
           if(workspaceMethod != null) {
               workspaceMethod.releaseConnection()
           }
        }
    }
    workspaceRef = getWorkspace()

    def isError = false
    for(defect in props['defectKey'].split(',')) {
        if (defect.trim().length() > 0) {
            HttpMethod queryMethod = new GetMethod(url + props['type'] + "?workspace=" + URLEncoder.encode(workspaceRef, "UTF-8") + "&query=" + URLEncoder.encode("(FormattedID = " + defect.trim() + ")" ,"UTF-8") + "&" + URLEncoder.encode("fetch", "UTF-8") + "=" + URLEncoder.encode("true", "UTF-8"))
            queryResult=client.executeMethod(queryMethod)
            queryResponse=new XmlParser().parseText(queryMethod.getResponseBodyAsString()).Results.Object.ObjectID[0].text()


            HttpMethod method = new PostMethod(url+props['type']+"/"+queryResponse)
            method.setRequestEntity((RequestEntity)new StringRequestEntity(writer.toString(), null, null));
            result = client.executeMethod(method)
            response = method.getResponseBodyAsString()


            //check results
            def changeResult = new XmlParser().parseText(response);
            def errors = changeResult.Errors.OperationResultError.size();
            def warnings = changeResult.Warnings.OperationResultWarning.size();

            if(errors == 0 && warnings == 0 && result >=200 && result <300) {
              System.out.println("Successfully updated item " + defect);
            }
            else {
              System.out.println("ERROR!!! Rally Server could not update item " + defect)
              changeResult.Errors.OperationResultError.each { System.out.println(it.text()) }
              changeResult.Warnings.OperationResultWarning.each { System.out.println(it.text()) }
              isError = true
            }
        }
    }


    if(isError) {
      System.exit 1
    }
}
else {
    println 'No issues found to update!'
}