import org.apache.commons.httpclient.*
import org.apache.commons.httpclient.methods.*;
import org.apache.commons.httpclient.protocol.*;
import org.apache.commons.httpclient.auth.AuthScope;
import com.urbancode.commons.util.https.OpenSSLProtocolSocketFactory


final def isWindows = (System.getProperty('os.name') =~ /(?i)windows/).find()
final def ahptool = isWindows ? 'ahptool.cmd' : 'ahptool'
final def out = System.out

final def getStepProperties = {
    def props = [:]

    def ahptoolProc = [ahptool, "getStepProperties"].execute()
    ahptoolProc.consumeProcessErrorStream(System.out);
    def propertiesXml = ahptoolProc.text
    ahptoolProc.waitFor()
    if (ahptoolProc.exitValue()) {
        throw new Exception("Failed to get step properties from ahptool: " + propertiesXml)
    }
    new XmlSlurper().parseText(propertiesXml).property.each{ propElem ->
        props[propElem.@name.text()] = propElem.text()
    }

    return props;
}

//-------------------------------------------------------------------------
// Scripts variables
//=========================================================================

final def props = getStepProperties()


System.out.println "Server:\t"+props['integration/rallyUrl']
System.out.println "Workspace Key:\t"+props['workspace']
System.out.println "Project Key:\t"+props['projectKey']
System.out.println "Requirement Key:\t"+props['reqKey']
System.out.println "Summary:\t"+ props['defectName']
            
//post to rally
String base_url = new URL(props['integration/rallyUrl']).getHost()
String url = props['integration/rallyUrl']+"/slm/webservice/"+props['integration/rallyVer']+"/"
String user = props['integration/rallyUser']
String password = props['integration/rallyPass']?props['integration/rallyPass']:props['integration/passScript']
String proxyHost = props['integration/proxyHost']
String proxyPort = props['integration/proxyPort']
String proxyPass = props['integration/proxyPass']
String proxyUser = props['integration/proxyUser']

// create and setup httpClient

HttpClient client = new HttpClient()
client.getParams().setAuthenticationPreemptive(true)
Credentials defaultcreds = new UsernamePasswordCredentials(user, password)
client.getState().setCredentials(new AuthScope(base_url, 443), defaultcreds)

ProtocolSocketFactory socketFactory = new OpenSSLProtocolSocketFactory()
Protocol https = new Protocol("https", socketFactory, 443)
Protocol.registerProtocol("https", https)

// setup proxy if neccessary
if (proxyHost) {
	client.getHostConfiguration().setProxy(proxyHost, Integer.valueOf(proxyPort));
	if (proxyUser && proxyPass) {
		client.getState().setProxyCredentials(AuthScope.ANY,
				new UsernamePasswordCredentials(proxyUser, proxyPass));
	}
}


def final getWorkspace = {
    HttpMethod workspaceMethod = null
    try {
        workspaceMethod= new GetMethod(url + "subscription")
        workResult = client.executeMethod(workspaceMethod)
        workResponse = workspaceMethod.getResponseBodyAsString()
        workXml = new XmlParser().parseText(workResponse)
        return workXml.Workspaces.Workspace.findAll{it.'@refObjectName'.equals(props['integration/workspace']) }[0].'@ref'
    }
    finally {
       if(workspaceMethod != null) {
           workspaceMethod.releaseConnection()
       }
    }
}
workspaceRef = getWorkspace()
//-----------------------------------------------------------------------------
// Heres where the magic happens
// =============================================================================

if(props['projectKey'] && props['projectKey'].length() > 0) {
  projectMethod = new GetMethod(url + "project?workspace=" + URLEncoder.encode(workspaceRef, "UTF-8") + "&query=" + URLEncoder.encode("(Name = " + props['projectKey'] + ")" ,"UTF-8") + "&" + URLEncoder.encode("fetch", "UTF-8") + "=" + URLEncoder.encode("true", "UTF-8"))
  client.executeMethod(projectMethod)
  projId = new XmlParser().parseText(projectMethod.getResponseBodyAsString()).Results.Object.ObjectID[0].text()
  if(projectMethod != null) {
     projectMethod.releaseConnection()
  }
}
else {
  requirementMethod = new GetMethod(url + "hierarchicalrequirement?workspace=" + URLEncoder.encode(workspaceRef, "UTF-8") + "&query=" + URLEncoder.encode("(FormattedID = " + props['reqKey'] + ")" ,"UTF-8") + "&" + URLEncoder.encode("fetch", "UTF-8") + "=" + URLEncoder.encode("true", "UTF-8"))
  client.executeMethod(requirementMethod)
  reqId = new XmlParser().parseText(requirementMethod.getResponseBodyAsString()).Results.Object.ObjectID[0].text()
  projId = new XmlParser().parseText(requirementMethod.getResponseBodyAsString()).Results.Object.Project.'@ref'[0]
  if(requirementMethod != null) {
     requirementMethod.releaseConnection()
  }
}
// Generate XML doc
def writer = new StringWriter() 
def builder = new groovy.xml.MarkupBuilder(writer) 
builder.Defect(){
  Project(projId)
  if(props['reqKey'] && props['reqKey'].length() >0) {
    Requirement(reqId)
  }
  Description(props['defectDesc'])
  Name(props['defectName'])
  Priority(props['priority'])
  Severity(props['severity'])
  State(props['state'])
  Owner(props['submitter'])
  Environment(props['env'])
  FoundInBuild(props['build'])
  //Workspace(workspaceRef)
}

  

HttpMethod method = new PostMethod(url + "defect/create")
method.setRequestEntity((RequestEntity)new StringRequestEntity(writer.toString(), null, null));
result = client.executeMethod(method)
response = method.getResponseBodyAsString()

//check results
def createResult = new XmlParser().parseText(response);
def errors = createResult.Errors.OperationResultError.size();
def warnings = createResult.Warnings.OperationResultWarning.size();

if(errors == 0 && warnings == 0) {
  System.out.println("Successfully created defect! ObjectRef =" + createResult.Object.'@ref');
  System.exit 0
}
else {
  System.out.println("ERROR!!! Rally Server could not create defect! ")
  createResult.Errors.OperationResultError.each { System.out.println(it.text()) }
  createResult.Warnings.OperationResultWarning.each { System.out.println(it.text()) }
  System.exit 1
}  