import org.apache.commons.httpclient.*
import org.apache.commons.httpclient.methods.*;
import org.apache.commons.httpclient.protocol.*;
import org.apache.commons.httpclient.auth.AuthScope;
import com.urbancode.commons.util.https.OpenSSLProtocolSocketFactory;


final def isWindows = (System.getProperty('os.name') =~ /(?i)windows/).find()
final def ahptool = isWindows ? 'ahptool.cmd' : 'ahptool'
final def out = System.out

final def getStepProperties = {
    def props = [:]

    def ahptoolProc = [ahptool, "getStepProperties"].execute()
    ahptoolProc.consumeProcessErrorStream(System.out);
    def propertiesXml = ahptoolProc.text
    ahptoolProc.waitFor()
    if (ahptoolProc.exitValue()) {
        throw new Exception("Failed to get step properties from ahptool: " + propertiesXml)
    }
    new XmlSlurper().parseText(propertiesXml).property.each{ propElem ->
        props[propElem.@name.text()] = propElem.text()
    }

    return props;
}

//-------------------------------------------------------------------------
// Scripts variables
//=========================================================================

final def props = getStepProperties()


System.out.println "Server:\t"+props['integration/rallyUrl']
System.out.println "Status:\t"+ props['status']

// Generate XML doc
def writer = new StringWriter() 
def builder = new groovy.xml.MarkupBuilder(writer) 
builder.Build(){
  BuildDefinition(props['buildDefId'])
  Duration(props['duration'])
  Label(props['label'])
  Message(props['message'])
  Status((props['status'].equals("Success")?'SUCCESS':'FAILURE'))
}


  
//post to rally
String base_url = new URL(props['integration/rallyUrl']).getHost()
String url = props['integration/rallyUrl']+"/slm/webservice/"+props['integration/rallyVer']+"/build/create"
String user = props['integration/rallyUser']
String password = props['integration/rallyPass']?props['integration/rallyPass']:props['integration/passScript']
String proxyHost = props['integration/proxyHost']
String proxyPort = props['integration/proxyPort']
String proxyPass = props['integration/proxyPass']
String proxyUser = props['integration/proxyUser']

// create and setup httpClient

HttpClient client = new HttpClient()
client.getParams().setAuthenticationPreemptive(true)
Credentials defaultcreds = new UsernamePasswordCredentials(user, password)
client.getState().setCredentials(new AuthScope(base_url, 443), defaultcreds)

ProtocolSocketFactory socketFactory = new OpenSSLProtocolSocketFactory()
Protocol https = new Protocol("https", socketFactory, 443)
Protocol.registerProtocol("https", https)

// setup proxy if neccessary
if (proxyHost) {
	client.getHostConfiguration().setProxy(proxyHost, Integer.valueOf(proxyPort));
	if (proxyUser && proxyPass) {
		client.getState().setProxyCredentials(AuthScope.ANY,
				new UsernamePasswordCredentials(proxyUser, proxyPass));
	}
}



HttpMethod method = new PostMethod(url)
method.setRequestEntity((RequestEntity)new StringRequestEntity(writer.toString(), null, null));
result = client.executeMethod(method)
response = method.getResponseBodyAsString()

//check results
def createResult = new XmlParser().parseText(response);
def errors = createResult.Errors.OperationResultError.size();
def warnings = createResult.Warnings.OperationResultWarning.size();

if(errors == 0 && warnings == 0 && result >=200 && result <300) {
  System.out.println("Successfully created Build! ObjectRef =" + createResult.Object.'@ref');
  System.exit 0
}
else {
   
  System.out.println("ERROR!!! Rally Server could not create Build! ")
  createResult.Errors.OperationResultError.each { System.out.println(it.text()) }
  createResult.Warnings.OperationResultWarning.each { System.out.println(it.text()) }
  System.exit 1
}  
